/**
 * This callback is called on a socket.on "corpus_analysis_send_meta_data".
 * Handels incoming corpus metadata
 */
function saveMetaData() {
  let [payload, client, results, rest] = arguments;
  results.metaData.init(payload)
  console.info('Metada saved:', results);
  client.isBusy = false;
}

/**
 * This callback should be registered to the SocketEventListener 'recieveQueryStatus'.
 * It just gets the incoming status data of the issued query
 * and does some preperation work like hiding or showing elements and deleting
 * the data from the last query.
 */
function prepareQueryData() {
  // deletes old data from query issued before this new query
  let [payload, client, results, rest] = arguments;
  // always initialize the results to delete data from the query issued before
  results.init();
  results.data.match_count = payload.match_count;
  client.requestQueryProgress = 0;
  client.notifyView('query-data-prepareing', { results: results });
}

/**
 * This callbacks saves the incoming query data chunks into the model results.
 */
function saveQueryData() {
  let [payload, client, results, rest] = arguments;
  let dataLength = results.data.matches.length;
  if (client.dynamicMode) {
    // get data matches length before new chunk data is being inserted
    // incorporating new chunk data into full results
    results.data.matches.push(...payload.chunk.matches);
    results.data.addData(payload.chunk.cpos_lookup, 'cpos_lookup');
    results.data.addData(payload.chunk.text_lookup, 'text_lookup');
    results.data.cpos_ranges = payload.chunk.cpos_ranges;
    let queryFormElement = document.querySelector('#query-form');
    results.data.getQueryStr(queryFormElement);
    client.requestQueryProgress = payload.progress;
    client.notifyView('query-data-recieving',
                      { results: results,
                        client: client,
                        dataLength: dataLength });
    console.info('Query data chunk saved', results.data);
    if (client.requestQueryProgress === 100) {
      client.isBusy = false;
      client.notifyView('query-data-recieved');
    }
  } else {
    results.data.matches.push(...payload.matches);
    results.data.addData(payload.cpos_lookup, 'cpos_lookup');
    results.data.addData(payload.text_lookup, 'text_lookup');
    results.data.cpos_ranges = payload.cpos_ranges;
    let queryFormElement = document.querySelector('#query-form');
    results.data.getQueryStr(queryFormElement);
    client.requestQueryProgress = 100;
    client.notifyView('query-data-recieving',
                      { results: results,
                        client: client,
                        dataLength: dataLength });
    console.info('Query data chunk saved', results.data);
    if (client.requestQueryProgress === 100) {
      console.log(results.data);
      client.notifyView('query-data-recieved');
    }
  }
}

function getResultsData() {
  let [resultsType, dataIndexes, client, results, rest] = arguments;
  client.isBusy = true;
  client.notifyView('results-data-recieving');
  client.getResultsData(resultsType, dataIndexes, results);
}

function saveResultsData() {
  let [payload, type, client, results, rest] = arguments;
  let objectKey = '';
  if (type === 'full-results') {
    console.info('Saving full-results data.');
    objectKey = 'fullResultsData';
  } else if (type === 'sub-results') {
    console.info('Saving sub-results data.');
    objectKey = 'subResultsData';
  } else if (type = 'inspect-results') {
    objectKey = 'inspectResultsData'
    console.info('Saving inspect-results data');
  }
  // Save incoming data
  results[objectKey].init();
  results[objectKey].matches.push(...payload.matches);
  results[objectKey].addData(payload.cpos_lookup, "cpos_lookup");
  results[objectKey].addData(payload.text_lookup, "text_lookup");
  results[objectKey].addData(results.metaData);
  results[objectKey].query = results.data.query;
  results[objectKey].corpus_type = type;
  results[objectKey].match_count = [...payload.matches].length;
  results[objectKey].cpos_ranges = payload.cpos_ranges;
  console.info('Results data has been saved.', results);
  client.isBusy = false;
  client.notifyView('results-data-recieved', {type: type,
                                              results: results});
}

// export callbacks
export {
  prepareQueryData,
  saveMetaData,
  saveQueryData,
  getResultsData,
  saveResultsData,
};