/**
 * This file contains all the callbacks triggered by the notificationListener.
 * Also general callbacks are defined which are doing some hiding/disabling and
 * showing/enabling of common elements when data is being transmitted or not.
 */

function disableElementsGeneralCallback(resultsList, detail) {
  if (detail.client.isBusy) {
    resultsList.fullResultsCreate.classList.toggle('disabled', true);
    resultsList.subResultsCreate.classList.toggle('disabled', true);
    resultsList.toggleInspectButtons(detail.client);
  }
}

function enableElementsGeneralCallback(resultsList, detail) {
  if (!detail.client.isBusy) {
    resultsList.fullResultsCreate.classList.toggle('disabled', false);
    if (resultsList.addToSubResultsIdsToShow.size > 0) {
      resultsList.subResultsCreate.classList.toggle('disabled', false);
    }
    resultsList.toggleInspectButtons(detail.client);
  }
}

function connectingCallback(resultsList, detail) {
  resultsList.getHTMLElements(['#analysis-init-modal']);
  resultsList.analysisInitModal = M.Modal.init(resultsList.analysisInitModal,
                                               {dismissible: false});
  resultsList.analysisInitModal.open();
}

function connectedCallback(resultsList, detail) {
  resultsList.analysisInitModal.close();
}

function clientFailedCallback(resultsList, detail) {
  resultsList.getHTMLElements([
    '#analysis-init-progress',
    '#analysis-init-error',
    '#user-feedback',
  ]);
  if (detail.raiseModalFeedback) {
    resultsList.analysisInitModal.open();
    resultsList.analysisInitProgress.classList.toggle('hide');
    resultsList.analysisInitError.classList.toggle('hide');
    resultsList.analysisInitError.textContent = detail.msg;
  } else {
    nopaque.flash(detail.msg, 'error')
  }
}

function queryDataPreparingCallback(resultsList, detail) {
  // remove all items from resultsList, like from the query issued before
  resultsList.clear()
  // get needed HTML Elements
  let results = detail.results;
  resultsList.getHTMLElements([
    '#interactions-menu',
    '#recieved-match-count',
    '#total-match-count',
    '#text-lookup-count',
    '#text-lookup-titles',
    '#query-results-user-feedback',
    '#query-progress-bar',
    '#query-results-create',
    '#sub-results-match-ids',
    '#nr-marked-matches',
   ]);
  // show or enable some things for the user
  resultsList.interactionsMenu.classList.toggle('hide', false)
  resultsList.queryResultsUserFeedback.classList.toggle('hide', false);
  resultsList.queryProgressBar.classList.toggle('hide', false);
  /**
   * Set some initial values for the user feedback
   * or reset values for new issued query
   */
  resultsList.recievedMatchCount.textContent = 0;
  resultsList.totalMatchCount.textContent = results.data.match_count;
  resultsList.textLookupTitles.textContent = '';
  resultsList.textLookupCount.textContent = 0;
  resultsList.nrMarkedMatches.textContent = 0;
  resultsList.subResultsMatchIds.textContent = '';
  resultsList.resetFields();
}

function queryDataRecievingCallback(resultsList, detail) {
  // load the data into the resultsList and show them to the user
  let results = detail.results;
  let client = detail.client;
  let start = detail.dataLength;
  let resultItems = [];
  for (let [index, match] of Object.entries(results.data.matches).slice(start)) {
    resultItems.push({ ...match, ...{ 'index': parseInt(index) } });
  }
  if (client.dynamicMode) {
    resultsList.add(resultItems, (items) => {
      for (let item of items) {
        item.elm = resultsList.createResultRowElement(item,
                                                      results.data,
                                                      client);
      }
    });
  // update user feedback about query status
  resultsList.recievedMatchCount.textContent = results.data.matches.length;
  resultsList.queryProgressBar.firstElementChild.style.width = `${client.requestQueryProgress}%`;
  resultsList.textLookupCount.textContent = `${Object.keys(results.data.text_lookup).length}`;
  let titles = new Array();
  for (let [key, value] of Object.entries(results.data.text_lookup)) {
    titles.push(`${value.title} (${value.publishing_year})`);
  }
  resultsList.textLookupTitles.textContent = `${titles.join(', ')}`;
  // updating table on finished item creation callback via createResultRowElement
  resultsList.update();
  resultsList.changeHitsPerPage(client, results);
  resultsList.changeContext();
  //activate expertMode of switch is checked
  if (resultsList.displayOptionsFormExpertMode.checked) {
    resultsList.expertModeOn('query-display', results);
  }
  } else if (!client.dynamicMode) {
    resultsList.add(resultItems, (items) => {
      for (let item of items) {
        console.log(item);
        item.elm = resultsList.createResultRowElement(item,
                                                      results.data,
                                                      client,
                                                      true);
        console.log(item);
      }
    });
    // update user feedback about query status
    resultsList.recievedMatchCount.textContent = results.data.matches.length;
    resultsList.queryProgressBar.firstElementChild.style.width = `${client.requestQueryProgress}%`;
    resultsList.textLookupCount.textContent = `${Object.keys(results.data.text_lookup).length}`;
    let titles = new Array();
    for (let [key, value] of Object.entries(results.data.text_lookup)) {
      titles.push(`${value.title} (${value.publishing_year})`);
    }
    resultsList.textLookupTitles.textContent = `${titles.join(', ')}`;
    // updating table on finished item creation callback via createResultRowElement
    resultsList.update();
    resultsList.changeHitsPerPage(client, results);
    resultsList.changeContext();
  }
}

function queryDataRecievedCallback(resultsList, detail) {
  // hide or disable some things for the user
  resultsList.queryResultsUserFeedback.classList.toggle('hide');
  resultsList.queryProgressBar.classList.toggle('hide');
  // reset bar progress for next query
  resultsList.queryProgressBar.firstElementChild.style.width = '0%';
}

function resultsDataRecievingCallback(resultsList, detail) {
}

function resultsDataRecievedCallback(resultsList, detail) {
  // create strings for create buttons depending on type
  const handleType = (keyPrefix, text) => {
    // hides the create element after results have been recieved and reset it
    resultsList[`${keyPrefix}Create`].classList.toggle('hide');
    resultsList[`${keyPrefix}Create`].textContent = `Create ${text}`;
    resultsList[`${keyPrefix}Create`].insertAdjacentHTML('beforeend',
    `<i class="material-icons left">build</i>`);
    // show and highlight export button
    resultsList[`${keyPrefix}Export`].classList.toggle('hide', false);
    resultsList[`${keyPrefix}Export`].classList.toggle('pulse', true);
    setTimeout(() => {
      resultsList[`${keyPrefix}Export`].classList.toggle('pulse', false);
      clearTimeout();
    }, 3000)
  }
  if (detail.type === 'full-results') {
    handleType('fullResults', 'Results');
  } else if (detail.type ==='sub-results') {
    handleType('subResults', 'Sub-Results');
  } else if (detail.type ==='inspect-results') {
    if (resultsList.addToSubResultsIdsToShow.size === 0) {
      /**
      * Prevent create sub results button from being activated if it is disabled
      * and no matches have been marked by the user for sub results creation.
      */
      resultsList.subResultsCreate.classList.toggle('disabled', true);
    }
    resultsList.showMatchContext(detail.results, detail.client);
  }
}

// export the callbacks
export {
  connectingCallback,
  connectedCallback,
  clientFailedCallback,
  queryDataPreparingCallback,
  queryDataRecievingCallback,
  queryDataRecievedCallback,
  resultsDataRecievingCallback,
  resultsDataRecievedCallback,
  disableElementsGeneralCallback,
  enableElementsGeneralCallback,
};